#ifndef _HARK__BEAMFORMING__BF_H__INCLUDED_
#define _HARK__BEAMFORMING__BF_H__INCLUDED_ 1

#include "BF_Matrix.h"

#include <vector>

namespace BF
{
    /* BF アルゴリズム種類 */
    enum BfType{
	BfUnknown = 0, BfDS, BfWDS, BfNull, BfILSE, BfLCMV, BfGJ, BfMSNR, BfGSS, BfGICA, BfGHDSS, BfML, 
    };

    /* ICA, HDSS で用いる非線形関数の種類 */
    enum NlFuncType {
	NlFuncUnknown = 0, NlFuncTanh, NlFuncCube,
    };

    class BF
    {
    public:
	static const double BF_EPS;

    public:
	BF(int numFreq, int samplingRate, double soundSpeed);
	virtual ~BF();

    public:
	virtual BfType getBfAlgorithmType() const = 0;

	virtual void updateSources(const std::vector<MicLocation> &micPositions, const std::vector<SourceLocation> &sourcePositions) = 0;
	virtual void executeBeamForming(const Matrix &inputSpectrum, Matrix &outputSpectrum);
	virtual void adaptSeparationMatrices() = 0;

	void calculateTransformFunctions(const std::vector<MicLocation> &micPositions, const std::vector<SourceLocation> &sourcePositions, std::vector<Matrix> &transformFunctions) const;
	void setTransformFunctionDatabase(const TransformFunctionDatabase &transformFunctionDatabase);

    public:
	/*== コンストラクタで初期化するのみ ==*/
	const int    numFreq;
	const int    samplingRate;
	const double soundSpeed;
	TransformFunctionDatabase  *transformFunctionDatabase;

	/*== 動的に変動 ==*/
	/* updateSources で更新 */
	std::vector<SourceLocation> sourcePositions;
	std::vector<MicLocation>    micPositions;
	std::vector<Matrix>         transformFunctions; /* size_is(numFreq), element_is(Matrix(numMic, numSrc)) */

	/* executeBeamForming で更新 */
	std::vector<Matrix>         covarianceMatrices;  /* size_is(numFreq), element_is(Matrix(numMic, numMic)) */

	/* updateSources, adaptSeparationMatrices で更新 */
	std::vector<Matrix>         separationMatrices; /* size_is(numFreq), element_is(Matrix(numSrc, numMic)) */

	/* executeBeamForming で更新 */
	Matrix                      inputSpectrum;      /* size_is(numMic, numFreq) */
	Matrix                      outputSpectrum;     /* size_is(numSrc, numFreq) */


    };
}

#endif
