#include <string>
#include <iostream>
#include <cstdlib>
#include <complex>
#include <cmath>

#include "BufferedNode.h"
#include "Vector.h"
#include "Matrix.h"
#include "Map.h"
#include "Source.h"

using namespace std;
using namespace FD;

// Node name
class TestCompare;

DECLARE_NODE(TestCompare);
/*Node
 *
 * @name TestCompare
 * @category HARK:BeamForming
 * @description Test BeamForming Node
 *
 * @input_name INPUT_Y1
 * @input_type Matrix<complex<float> >
 * @input_description
 *
 * @input_name INPUT_Y2
 * @input_type Map<int,ObjectRef>
 * @input_description
 *
 * @output_name OUTPUT
 * @output_type string
 * @output_description
 *
 END*/

class TestCompare : public BufferedNode
{
public:
    TestCompare(string nodeName, ParameterSet params)
	: BufferedNode(nodeName, params)
    {
	/*== FlowDesigner 入出力インタフェース ==*/

	/* 入出力IDの割り当て */
	this->inputY1_ID = addInput("INPUT_Y1");
	this->inputY2_ID = addInput("INPUT_Y2");

	this->output_ID = addOutput("OUTPUT");
    }

    virtual ~TestCompare()
    {
    }

public:
    void calculate(int output_id, int count, Buffer &out)
    {
	ObjectRef orInputY1Temp = getInput(this->inputY1_ID, count);
	Matrix<complex<float> > &inputY1ValueMatrix = object_cast<Matrix<complex<float> > >(orInputY1Temp);

	ObjectRef orInputY2Temp = getInput(this->inputY2_ID, count);
	Map<int, ObjectRef> &inputY2ValueMatrix = object_cast<Map<int, ObjectRef> >(orInputY2Temp);

	const Matrix<complex<float> > &input  = inputY1ValueMatrix;
	Map<int, ObjectRef>::iterator it;

#if 1
	cout << "Y1: " << endl;
	for(int i = 0; i < inputY1ValueMatrix.nrows(); i++){
	    for(int j = 0; j < inputY1ValueMatrix.ncols(); j++){
		if(j == 0){
		    cout << "  [";
		}else{
		    cout << ", ";
		}
		cout << inputY1ValueMatrix(i, j);
	    }
	    cout << endl;
	}

	cout << "Y2: " << endl;
	for(it = inputY2ValueMatrix.begin(); it != inputY2ValueMatrix.end(); it++){
	    ObjectRef &o = (*it).second;
	    cout << o << endl;
	}
#endif

	for(it = inputY2ValueMatrix.begin(); it != inputY2ValueMatrix.end(); it++){
	    int        i = (*it).first;
	    ObjectRef &o = (*it).second;

	    Vector<complex<float> > output = object_cast<Vector<complex<float> > > (o);
	    
	    double norm1 = 0, norm2 = 0, normd = 0;
	    for(int j = 0; j < output.vsize(); j++){
		norm1 += norm(input(i, j));
		norm2 += norm(output[j]);
		normd += norm(input(i, j) - output[j]);
	    }

	    fprintf(stdout, "Frame#%d / Source#%d: |bf|=%f, |ml|=%f, |bf - ml|=%f\n", count, i, sqrt(norm2), sqrt(norm1), sqrt(normd));
	}

	(*(outputs[this->output_ID].buffer))[count] = ObjectRef(new String("hoge"));
    }

private:
    int inputY1_ID, inputY2_ID, output_ID;

};
